package com.takensoft.cms.bbs.service.Impl;

import com.takensoft.cms.bbs.dao.BbsCnDAO;
import com.takensoft.cms.bbs.dao.BbsMngDAO;
import com.takensoft.cms.bbs.dao.WordMngDAO;
import com.takensoft.cms.bbs.service.BbsCnService;
import com.takensoft.cms.bbs.vo.BbsCnVO;
import com.takensoft.cms.bbs.vo.BbsMngVO;
import com.takensoft.cms.bbs.vo.WordMngVO;
import com.takensoft.common.Pagination;
import com.takensoft.common.file.dao.FileDAO;
import com.takensoft.common.file.service.FileMngService;
import com.takensoft.common.file.vo.FileMngVO;
import com.takensoft.common.idgen.service.IdgenService;
import com.takensoft.common.util.JWTUtil;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author  : 하석형
 * @since   : 2024.05.09
 *
 * 게시판 내용 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * BbsCnService : 게시판 내용 인터페이스 상속
 */
@Service("bbsCnService")
@RequiredArgsConstructor
public class BbsCnServiceImpl extends EgovAbstractServiceImpl implements BbsCnService {
    private final BbsCnDAO bbsCnDAO;
    private final BbsMngDAO bbsMngDAO;
    private final FileDAO fileDAO;
    private final WordMngDAO wordMngDAO;

    private final IdgenService bbsIdgn;
    private final JWTUtil jwtUtil;

    private final FileMngService fileMngService;

    @Value("${front.url}")
    private String frontUrl;

    @Value("${server.port}")
    private String serverPort;

    @Value("${spring.web.resources.static-locations}")
    private String locationPath;

    @Value("${file.file-upload-path}")
    private String fileUploadPath;

    /**
     * @author  하석형
     * @since   2024.05.09
     * 게시판 아이디 중복 검사
     */
    @Override
    public boolean bbsCnIdCheck(BbsCnVO bbsCnVO) throws Exception {
        return bbsCnDAO.bbsCnIdCheck(bbsCnVO);
    }

    /**
     * @author  하석형
     * @since   2024.05.09
     * 게시판 내용 등록
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public HashMap<String, Object> saveBbsCn(BbsCnVO bbsCn, List<MultipartFile> multipartFileList, List<MultipartFile> multipartImgList) throws Exception {
        HashMap<String, Object> result = new HashMap<>();
        List<String> wordList = wordMngDAO.findWordList();

        // 금지어 포함 여부 체크
        // 제목에 금지어가 포함되어있는가?
        if(bbsCn.getBbsNm() != null && !bbsCn.getBbsNm().isEmpty()) {
        String bbsNm = bbsCn.getBbsNm();
        for (String word : wordList) {
            if (bbsNm.contains(word)) {
                result.put("result", -1);
                result.put("word", word);
                return result;
            }
        }
        }

        // 내용에 금지어가 포함되어있는가?
        String content = bbsCn.getBbsCn();
        for (String word : wordList) {
            if (content.contains(word)) {
                result.put("result", -1);
                result.put("word", word);
                return result;
            }
        }


        // 게시판 내용 아이디 생성
        String bbsId = bbsIdgn.getNextStringId();
        bbsCn.setBbsId(bbsId);

        // 등록된 토큰에서 사용자 정보 조회
        String writer = jwtUtil.getWriter();
        bbsCn.setRgtr(writer);

        // youtube 링크 여부에 따라 처리
        if(bbsCn.getVdoUrl() != null){
            bbsCn.setVdoUrl(vdoUrlCheck(bbsCn.getVdoUrl()));
        }


        // 첨부파일 유무
        if (multipartFileList != null && !multipartFileList.isEmpty()) {
            try {
                FileMngVO fileMngVO = new FileMngVO();

                // 파일매니저 등록
                HashMap<String, Object> fileMng = fileMngService.fileMngInsert(fileMngVO, multipartFileList);
                result.put("multipartFileMngResult", fileMng);

                // 파일매니저 아이디 삽입
                bbsCn.setAtchFileMngId(fileMng.get("fileMngId").toString());
            } catch (IOException e) {
                e.printStackTrace();
                return result;
            }
        }
        // 이미지파일 유무
        if (multipartImgList != null && !multipartImgList.isEmpty()) {
            try {
                FileMngVO fileMngVO = new FileMngVO();

                // 파일매니저 등록
                HashMap<String, Object> imgFileMng = fileMngService.fileMngInsert(fileMngVO, multipartImgList);
                result.put("multipartFileMngResult", imgFileMng);

                // 이미지 파일매니저 아이디 삽입
                bbsCn.setImgFileMngId(imgFileMng.get("fileMngId").toString());
            } catch (IOException e) {
                e.printStackTrace();
                return result;
            }
        }

        // 질의형인 경우 \n을 <br>로 변경
        if(bbsCn.getBbsNm() == null){
            String text = bbsCn.getBbsCn().replaceAll("\n", "<br/>");
            bbsCn.setBbsCn(text);
        }

        // 기본정보 DB 등록
        int saveBbsCnResult = bbsCnDAO.save(bbsCn);
        result.put("bbsId", bbsId);
        result.put("result", saveBbsCnResult);

        return result;
    }

    /**
     * @author  하석형
     * @since   2024.05.13
     * 게시판 내용 목록 조회 개수
     */
    public int findAllCnt(Pagination pagination) throws Exception{
        return bbsCnDAO.findAllCnt(pagination);
    }

    /**
     * @author  하석형
     * @since   2024.05.09
     * 게시판 내용 목록 조회
     */
    @Override
    public Map<String, Object> findAllBbsCn(HashMap<String, String> params) throws Exception {
        Map<String, Object> result = new HashMap<>();

        // 게시판 관리 정보
        BbsMngVO bbsMng = bbsMngDAO.findByBbsMngId(params.get("id"));
        params.put("recordSize", String.valueOf(bbsMng.getArtclCnt()));

        // 게시판 보여줄 갯수 설정
        Pagination search = new Pagination(0, params);
        int cnt = bbsCnDAO.findAllCnt(search);
        Pagination pagination = new Pagination(cnt, params);
        List<BbsCnVO> list = bbsCnDAO.findAll(pagination);

        list.forEach(entity -> {
            String imagePath = entity.getAbsltPath();
            entity.setAbsltPath(convertPathToUrl(imagePath));
        });

        result.put("list", list);
        result.put("pagination", pagination);
        result.put("codeList", null); // 검색 조건
        result.put("bbsMng", bbsMng); // 게시판 관리 정보
        return result;
    }

    private String convertPathToUrl(String filePath) {

        String previewPath = null;
        String[] pathArray = locationPath.split("/, ");
        for (String path : pathArray) {
            if (path.startsWith("file:///")) {
                // "file:///" 부분을 제거하고 경로를 반환
                previewPath = path.replace("file:///", "") + fileUploadPath;
            } else {
                previewPath = locationPath + fileUploadPath;
            }
        }
        if (filePath == null) return null;
        // 서버경로를 절대경로에서 상대경로로 변환
        String basePath = previewPath; // 이 부분은 환경설정 파일에서 가져오는 것이 좋습니다.
        String relativePath = filePath.substring(basePath.length());
        return frontUrl + ":" + serverPort + fileUploadPath + relativePath;
    }


    /**
     * @author 하석형
     * @since  2024.05.09
     *
     * 게시판 내용 상세 조회
     */
    @Override
    public BbsCnVO findByBbsId(String bbsId) throws Exception {
        BbsCnVO result = new BbsCnVO();
        if(bbsId != null && !bbsId.isEmpty()) {
            result = bbsCnDAO.findByBbsId(bbsId);

            if(result.getBbsNm() == null){
                String text = result.getBbsCn().replaceAll("<br/>", "\n");
                result.setBbsCn(text);
                if(result.getAnsCn() != null){
                    String ansText = result.getAnsCn().replaceAll("<br/>", "\n");
                    result.setAnsCn(ansText);
                }
            }

        } else {
            result.setUseYn("Y");
        }
        return result;
    }

    /**
     * @author 하석형
     * @since  2024.05.09
     *
     * 게시판 내용 수정
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public HashMap<String, Object> updateBbsCn(HashMap<String, Object> params, List<HashMap<String, Object>> deleteFileList, List<MultipartFile> multipartFileList, List<HashMap<String, Object>> deleteImgFileList, List<MultipartFile> multipartImgList) throws Exception {
        HashMap<String, Object> result = new HashMap<>();
        int intResult= 0;

        List<String> wordList = wordMngDAO.findWordList();

        // 금지어 포함 여부 체크
        // 제목에 금지어가 포함되어있는가?
        String bbsNm = (params.get("bbsNm") != null) ? params.get("bbsNm").toString() : null;
        if(bbsNm != null && !bbsNm.isEmpty()) {
            for (String word : wordList) {
                if (bbsNm.contains(word)) {
                    result.put("result", -1);
                    result.put("word", word);
                    return result;
                }
            }
        }

        // 내용에 금지어가 포함되어있는가?
        String content = params.get("bbsCn").toString();
        for (String word : wordList) {
            if (content.contains(word)) {
                result.put("result", -1);
                result.put("word", word);
                return result;
            }
        }

        // 등록된 토큰에서 사용자 정보 조회
        String writer = jwtUtil.getWriter();
        params.put("mdfr", writer);

        // youtube 링크 여부에 따라 처리
        if(params.get("vdoUrl") != null){
            params.put("vdoUrl", vdoUrlCheck(params.get("vdoUrl").toString()));
        }

        // 파일 업로드
        FileMngVO fileMngVO = new FileMngVO();
        if (params.get("atchFileMngId") != null) {
            // 파일매니저 수정
            try {
                String fileMngId = params.get("atchFileMngId").toString();
                fileMngVO.setFileMngId(fileMngId);
                intResult += fileMngService.fileMngUpadate(fileMngVO, deleteFileList, multipartFileList);
            } catch (IOException e) {
                e.printStackTrace();
                result.put("result", 0);
                return result;
            }
        } else if (multipartFileList != null && !multipartFileList.isEmpty()){
            // 파일매니저 등록
            try {
                HashMap<String, Object> fileMng = fileMngService.fileMngInsert(fileMngVO, multipartFileList);
                intResult += (int)fileMng.get("fileMngResult");
                params.put("atchFileMngId", fileMng.get("fileMngId").toString());
            } catch (IOException e) {
                e.printStackTrace();
                result.put("result", 0);
                return result;
            }
        }

        // 이미지 파일 업로드
        FileMngVO fileImgMngVO = new FileMngVO();
        if (params.get("imgFileMngId") != null) {
            // 파일매니저 수정
            try {
                String imgFileMng = params.get("imgFileMngId").toString();
                fileImgMngVO.setFileMngId(imgFileMng);
                intResult += fileMngService.fileMngUpadate(fileImgMngVO, deleteImgFileList, multipartImgList);
            } catch (IOException e) {
                e.printStackTrace();
                result.put("result", 0);
                return result;
            }
        } else if (multipartImgList != null && !multipartImgList.isEmpty()){
            // 파일매니저 등록
            try {
                HashMap<String, Object> imgFileMng = fileMngService.fileMngInsert(fileImgMngVO, multipartImgList);
                intResult += (int)imgFileMng.get("fileMngResult");
                params.put("imgFileMngId", imgFileMng.get("fileMngId").toString());
            } catch (IOException e) {
                e.printStackTrace();
                result.put("result", 0);
                return result;
            }
        }

        // 질의형인 경우 \n을 <br>로 변경
        if(params.get("bbsNm") == null){
            String text = params.get("bbsCn").toString().replaceAll("\n", "<br/>");
            params.put("bbsCn", text);
        }

        // 게시판 내용 수정
        intResult += bbsCnDAO.update(params);

        result.put("result", intResult);
        return result;
    }

    /**
     * @author 하석형
     * @since  2024.05.09
     *
     * 게시판 내용 삭제
     */
    @Override
    public int deleteBbsCn(BbsCnVO bbsCnVO) throws Exception {
        // 등록된 토큰에서 사용자 정보 조회
        String writer = jwtUtil.getWriter();
        bbsCnVO.setMdfr(writer);

        return bbsCnDAO.delete(bbsCnVO);
    }
    /**
     * @author 방선주
     * @since  2024.05.21
     *
     * vdoUrl 조건 처리
     */
     private String vdoUrlCheck(String vdoUrl) {
         // youtube 링크가 있을 경우
         if (vdoUrl.contains("youtube.com") || vdoUrl.contains("youtu.be")) {
             String[] url = vdoUrl.split("/");
             String videoId = url[url.length - 1];
             String videoUrl = "https://www.youtube.com/embed/" + videoId;
             return videoUrl;
         } else {
             return vdoUrl;
         }
     }

    /**
     * @author 방선주
     * @since  2024.05.22
     *
     * 조회수 증가 처리
     */
    @Override
    public int updateVwCnt(HashMap<String, Object> params) throws Exception {
        return bbsCnDAO.updateVwCnt(params.get("bbsId").toString());
    }

    /**
     * @author 방선주
     * @since  2024.05.23
     *
     * 이전글 다음글 조회
     *
     */
    @Override
    public BbsCnVO findPrevBbsCn(BbsCnVO bbsCnVO) throws Exception {
        Map<String, Object> result = new HashMap<>();

        BbsCnVO prevBbsCn = bbsCnDAO.findPrevBbsCn(bbsCnVO);
        return prevBbsCn;
    }
    /**
     * @author 방선주
     * @since  2024.05.23
     *
     * 이전글 다음글 조회
     *
     */
    @Override
    public BbsCnVO findNextBbsCn(BbsCnVO bbsCnVO) throws Exception {
        Map<String, Object> result = new HashMap<>();

        BbsCnVO NextBbsCn = bbsCnDAO.findNextBbsCn(bbsCnVO);
        return NextBbsCn;
    }

    /**
     * 첨부파일 이미지 경로 변경
     * @param fileMngId
     * @return
     * @throws Exception
     */
    @Override
    public List<HashMap<String, Object>> fileListPathChange(String fileMngId) throws Exception {

        List<HashMap<String, Object>> fileList = fileDAO.fileSelectList(fileMngId);

        for (HashMap<String, Object> file : fileList) {
            String absltPath = file.get("absltPath").toString();
            file.put("absltPath", convertPathToUrl(absltPath));
        }
        return fileList;
    }

    /**
     * 질의형 게시판 답변 등록
     * @param bbsCnVO
     * @return
     * @throws Exception
     */
    @Override
    public HashMap<String, Object> saveAnswer(BbsCnVO bbsCnVO) throws Exception {
        HashMap<String, Object> result = new HashMap<>();
        // 등록된 토큰에서 사용자 정보 조회
        String writer = jwtUtil.getWriter();
        bbsCnVO.setAnswer(writer);

        // 답변 금지어 포함 여부 체크
        List<String> wordList = wordMngDAO.findWordList();
        String ansCn = bbsCnVO.getAnsCn();
        for (String word : wordList) {
            if (ansCn.contains(word)) {
                result.put("result", -1);
                result.put("word", word);
                return result;
            }
        }

        // 질의형인 경우 \n을 <br>로 변경
        if(bbsCnVO.getBbsNm() == null){
            String text = bbsCnVO.getAnsCn().replaceAll("\n", "<br/>");
            bbsCnVO.setAnsCn(text);
        }

        int saveAns = bbsCnDAO.saveAnswer(bbsCnVO);

        result.put("result", saveAns);
        return result;
    }

    /**
     * 질의형 게시판 답변 수정
     * @param bbsCnVO
     * @return
     * @throws Exception
     */
    @Override
    public HashMap<String, Object> updateAnswer(BbsCnVO bbsCnVO) throws Exception {
        HashMap<String, Object> result = new HashMap<>();

        // 등록된 토큰에서 사용자 정보 조회
        String writer = jwtUtil.getWriter();
        bbsCnVO.setAnswer(writer);

        // 답변 금지어 포함 여부 체크
        List<String> wordList = wordMngDAO.findWordList();
        String ansCn = bbsCnVO.getAnsCn();
        for (String word : wordList) {
            if (ansCn.contains(word)) {
                result.put("result", -1);
                result.put("word", word);
                return result;
            }
        }

        String text = bbsCnVO.getAnsCn().replaceAll("\n", "<br/>");
        bbsCnVO.setAnsCn(text);

        int saveAns = bbsCnDAO.saveAnswer(bbsCnVO);

        result.put("result", saveAns);
        return result;
    }

    /**
     * 질의형 게시판 답변 삭제
     * @param bbsCnVO
     * @return
     * @throws Exception
     */
    @Override
    public int deleteAns(BbsCnVO bbsCnVO) throws Exception {
        // 등록된 토큰에서 사용자 정보 조회
        String writer = jwtUtil.getWriter();
        bbsCnVO.setAnswer(writer);

        bbsCnVO.setAnsCn(null);
        bbsCnVO.setAnsStts("N");

        return bbsCnDAO.saveAnswer(bbsCnVO);
    }

    /**
     * 최근 공지사항 5개 가져오기
     * @param
     * @return
     * @throws Exception
     */
    @Override
    public List<BbsCnVO> findFiveNotice(BbsCnVO bbsCn) throws Exception{
        return bbsCnDAO.findFiveNotice(bbsCn);
    }


}