package com.takensoft.cms.bbs.service.Impl;

import com.takensoft.cms.bbs.dao.BbsMngDAO;
import com.takensoft.cms.bbs.service.BbsCnService;
import com.takensoft.cms.bbs.service.BbsMngService;
import com.takensoft.cms.bbs.service.BbsTypeMngService;
import com.takensoft.cms.bbs.vo.BbsMngVO;
import com.takensoft.cms.bbs.vo.BbsRouterVO;
import com.takensoft.cms.bbs.vo.BbsTypeMngVO;
import com.takensoft.cms.codeManage.service.CodeManageService;
import com.takensoft.cms.menu.service.MenuService;
import com.takensoft.cms.menu.vo.MenuVO;
import com.takensoft.common.HierachyVO;
import com.takensoft.common.Pagination;
import com.takensoft.common.idgen.service.IdgenService;
import com.takensoft.common.util.JWTUtil;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.05.08
 *
 * 게시판 관리 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * BbsMngService : 게시판 관리 인터페이스 상속
 */
@Service("bbsMngService")
@RequiredArgsConstructor
public class BbsMngServiceImpl extends EgovAbstractServiceImpl implements BbsMngService {
    private final BbsMngDAO bbsMngDAO;
    private final BbsTypeMngService bbsTypeMngService;
    private final BbsCnService bbsCnService;
    private final IdgenService bbsMngIdgn;
    private final IdgenService bbsRouterIdgn;
    private final MenuService menuService;

    private final JWTUtil jwtUtil;

    /**
     * @author  박정하
     * @since   2024.05.08
     * 게시판 아이디 중복 검사
     */
    @Override
    public boolean bbsMngIdCheck(BbsMngVO bbsMngVO) throws Exception {
        return bbsMngDAO.bbsMngIdCheck(bbsMngVO);
    }

    /**
     * @author  하석형
     * @since   2024.05.10
     * 게시판 관리 등록
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    @CacheEvict(value = "routerAuthInfo", allEntries = true)
    public HashMap<String, Object> saveBbsMng(BbsMngVO bbsMngVO) throws Exception {
        HashMap<String, Object> result = new HashMap<>();

        // 게시판 관리 아이디 생성
        String bbsMngId = bbsMngIdgn.getNextStringId();
        bbsMngVO.setBbsMngId(bbsMngId);

        // 등록된 토큰에서 사용자 정보 조회
        String writer = jwtUtil.getWriter();
        bbsMngVO.setRgtr(writer);

        // 확장자 리스트 문자열로 변경
        bbsMngVO.setFileExtnNm(String.join(",", bbsMngVO.getFileExtnNmList()));

        // 첨부파일 용량 변경 (Mb -> Byte)
        bbsMngVO.setFileSzLmt(bbsMngVO.getFileSzLmt() * 1024 * 1024);

        // 게시판 관리 등록
        int insertResult = bbsMngDAO.save(bbsMngVO);

        // 게시판 라우터 등록 프로세스
        saveBbsRouterProc(bbsMngVO);

        // 메뉴 등록 MENU_000000000000032 : 게시판 관리 메뉴 아이디

        MenuVO menuVO = new MenuVO();
        menuVO.setUpMenuId("MENU_000000000000032"); // 상위메뉴아이디(콘텐츠 관리 메뉴 아이디)
        menuVO.setMenuNm(bbsMngVO.getBbsNm()); // 메뉴명
        menuVO.setMenuType("bbs"); // 메뉴타입
        menuVO.setMenuTypeCtgry(bbsMngVO.getBbsMngId()); // 메뉴타입항목(콘텐츠 유형 아이디)
        menuVO.setLinkType("0"); // 링크유형(현재창)
        menuVO.setDgstfnExmnUseYn("Y"); // 만족도조사사용여부(노출)
        menuVO.setMenuExpsrYn("Y"); // 메뉴노출여부(노출)
        menuVO.setMenuUser("A"); // 메뉴사용자(관리자)
        menuVO.setSysPvsnYn("1"); // 시스템제공여부(시스템등록)
        menuService.menuSave(menuVO);

        result.put("bbsMngId", bbsMngId);
        result.put("result", insertResult);
        return result;
    }

    /**
     * @author  하석형
     * @since   2024.05.10
     * 게시판 관리 목록 조회
     */
    @Override
    public Map<String, Object> findAllBbsMng(HashMap<String, String> params) throws Exception {
        Map<String, Object> result = new HashMap<>();
        Pagination search = new Pagination(0, params);
        int cnt = bbsMngDAO.findAllCnt(search);
        Pagination pagination = new Pagination(cnt, params);
        List<BbsMngVO> bbsMngList = bbsMngDAO.findAll(pagination);

        List<HashMap<String, Object>> list = new ArrayList<>();

        //게시판 관리 목록 별 게시물 수
        for(BbsMngVO vo : bbsMngList) {
            HashMap<String, Object> map = new HashMap<>();

            // 게시판 유형 그룹
            BbsTypeMngVO bbsTypeMng = bbsTypeMngService.findByBbsTypeId(vo.getBbsTypeId());
            String bbsTypeNm = bbsTypeMng.getBbsTypeKornNm();
            int spaceIndex = bbsTypeNm.indexOf(" "); // 첫 번째 공백의 인덱스를 찾음
            if (spaceIndex != -1) { // 공백이 있는 경우
                bbsTypeNm = bbsTypeNm.substring(0, spaceIndex); // 첫 공백 전까지의 문자열을 추출
            }

            map.put("bbsMngId", vo.getBbsMngId());
            map.put("bbsNm", vo.getBbsNm());
            map.put("bbsTypeId", vo.getBbsTypeId());
            map.put("bbsTypeNm", bbsTypeNm);
            map.put("atchFileUseYn", vo.getAtchFileUseYn());
            map.put("ntcUseYn", vo.getNtcUseYn());
            map.put("prvtPstUseYn", vo.getPrvtPstUseYn());
            map.put("bfrAftrPstUseYn", vo.getBfrAftrPstUseYn());
            map.put("cmntUseYn", vo.getCmntUseYn());
            map.put("bbsCnCnt", vo.getBbsCnCnt()); // 게시판 관리 목록 별 게시물 수 (수정 필요)
            list.add(map);
        }

        result.put("list", list);
        result.put("pagination", pagination);
        return result;
    }

    /**
     * @author  박정하
     * @since   2024.05.16
     * 게시판 관리 목록 조회 (메뉴 관리용)
     */
    @Override
    public List<BbsMngVO> findAllByMenuMng() throws Exception {
        return bbsMngDAO.findAllByMenuMng();
    }

    /**
     * @author 하석형
     * @since  2024.05.08
     *
     * 게시판 관리 상세 조회
     */
    @Override
    public BbsMngVO findByBbsMngId(String bbsMngId) throws Exception {
        BbsMngVO result = new BbsMngVO();

        if(bbsMngId != null && !bbsMngId.isEmpty()) {
            result = bbsMngDAO.findByBbsMngId(bbsMngId);
            // 첨부파일 확장자 리스트 분리
            if(result.getFileExtnNm() != null && !result.getFileExtnNm().isEmpty()){
                result.setFileExtnNmList(Arrays.asList(result.getFileExtnNm().split(",")));
            }
            // 첨부파일 용량 변경 (Byte -> Mb)
            result.setFileSzLmt(result.getFileSzLmt() / 1024 / 1024);

        } else {
            result.setUseYn("Y");
        }
        return result;
    }

    /**
     * @author 하석형
     * @since  2024.05.09
     *
     * 게시판 관리 수정
     */
    @Override
    @CacheEvict(value = "routerAuthInfo", allEntries = true)
    public int updateBbsMng(BbsMngVO bbsMngVO) throws Exception {
        // 등록된 토큰에서 사용자 정보 조회
        String writer = jwtUtil.getWriter();
        bbsMngVO.setMdfr(writer);

        // 확장자 리스트 문자열로 변경
        bbsMngVO.setFileExtnNm(String.join(",", bbsMngVO.getFileExtnNmList()));

        bbsMngVO.setFileSzLmt(bbsMngVO.getFileSzLmt() * 1024 * 1024);

        /** 게시판 라우터 수정 프로세스 */
        BbsMngVO bbsMng = bbsMngDAO.findByBbsMngId(bbsMngVO.getBbsMngId());

        // 게시판 유형이 변경된 경우
        if(!bbsMng.getBbsTypeId().equals(bbsMngVO.getBbsTypeId())) {
            // 기존 게시판 라우터 삭제
            deleteBbsRouter(bbsMngVO.getBbsMngId());
            
            // 변경된 유형으로 게시판 라우터 등록
            saveBbsRouterProc(bbsMngVO);
        }
        // 게시판 페이지 유형이 변경된 경우
        if(!bbsMng.getCd().equals(bbsMngVO.getCd())) {
            // 게시판 라우터 페이지 경로 변경
            List<BbsRouterVO> bbsRouterList = bbsMngDAO.findAllBbsRouter(bbsMngVO.getBbsMngId());
            for(BbsRouterVO route : bbsRouterList) {
                String pageCrs = route.getPageCrs().replace(bbsMng.getCd(), bbsMngVO.getCd());
                route.setPageCrs(pageCrs);
                bbsMngDAO.updateCrs(route);

            }

        }

        return bbsMngDAO.update(bbsMngVO);
    }

    /**
     * @author 하석형
     * @since  2024.05.09
     *
     * 게시판 관리 삭제
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    @CacheEvict(value = "routerAuthInfo", allEntries = true)
    public int deleteBbsMng(BbsMngVO bbsMngVO) throws Exception {
        // 등록된 토큰에서 사용자 정보 조회
        String writer = jwtUtil.getWriter();
        bbsMngVO.setMdfr(writer);
        
        // 게시판 라우터 비활성화
        BbsRouterVO bbsRouterVO = new BbsRouterVO();
        bbsRouterVO.setBbsMngId(bbsMngVO.getBbsMngId());
        bbsRouterVO.setUseYn("N");
        updateBbsRouter(bbsRouterVO);

        // 메뉴에서도 삭제 처리
        MenuVO menuVO = new MenuVO();
        menuVO.setMenuTypeCtgry(bbsMngVO.getBbsMngId());
        menuVO.setUseYn("N");
        menuVO.setMdfr(writer);
        int bbsMngMenuResult = menuService.bbsMngMenuDelete(menuVO);



        return bbsMngDAO.delete(bbsMngVO);
    }


    /** ********************************* 게시판 라우터 (시작) ********************************* */
    /**
     * @author 하석형
     * @since  2024.05.13
     *
     * 게시판 라우터 등록
     */
    @Override
    @CacheEvict(value = "routerAuthInfo", allEntries = true)
    public int saveBbsRouter(BbsRouterVO bbsRouterVO) throws Exception {
        return bbsMngDAO.saveBbsRouter(bbsRouterVO);
    }

    /**
     * @author 하석형
     * @since  2024.05.13
     *
     * 게시판 라우터 수정
     */
    @Override
    @CacheEvict(value = "routerAuthInfo", allEntries = true)
    public int updateBbsRouter(BbsRouterVO bbsRouterVO) throws Exception {
        return bbsMngDAO.updateBbsRouter(bbsRouterVO);
    }

    /**
     * @author 하석형
     * @since  2024.05.13
     *
     * 게시판 라우터 삭제
     */
    @Override
    @CacheEvict(value = "routerAuthInfo", allEntries = true)
    public int deleteBbsRouter(String bbsMngId) throws Exception {
        return bbsMngDAO.deleteBbsRouter(bbsMngId);
    }

    /**
     * @author  하석형
     * @since   2024.05.13
     * 게시판 라우터 등록 프로세스
     */
    @Transactional(rollbackFor = Exception.class)
    @CacheEvict(value = "routerAuthInfo", allEntries = true)
    public void saveBbsRouterProc(BbsMngVO bbsMngVO) throws Exception {
        int result = 0;
        try {
            // 게시판 유형 조회 (bbs_type 조회)
            BbsTypeMngVO bbsTypeMng = bbsTypeMngService.findByBbsTypeId(bbsMngVO.getBbsTypeId());
            // 동일한 유형의 게시판 유형 목록 조회
            List<BbsTypeMngVO> bbsTypeList = (List<BbsTypeMngVO>) bbsTypeMngService.findAllBbsTypeMng(bbsTypeMng).get("list");
            // 게시판 유형 목록만큼 게시판 라우터 등록
            for (BbsTypeMngVO vo : bbsTypeList) {
                // 빈 게시판 라우터 VO 생성
                BbsRouterVO bbsRouterVO = new BbsRouterVO();

                // 게시판 라우터 아이디 생성
                String bbsRouterId = bbsRouterIdgn.getNextStringId();

                // 페이지 경로에 게시판 관리 아이디 추가
                String pageCrs = vo.getPageCrs().replace("change", bbsMngVO.getBbsMngId());
                String userPageCrs = pageCrs.replace("cmmn", bbsMngVO.getCd());
                String adminPageCrs = pageCrs.replace("cmmn", "adm");
                String compnCrs = vo.getCompnCrs();

                bbsRouterVO.setBbsRouterId(bbsRouterId);
                bbsRouterVO.setBbsMngId(bbsMngVO.getBbsMngId());
                bbsRouterVO.setBbsKornNm(vo.getBbsTypeKornNm());
                bbsRouterVO.setBbsEngNm(vo.getBbsTypeEngNm() + "_" + bbsMngVO.getBbsMngId());
                bbsRouterVO.setPageCrs(userPageCrs);
                bbsRouterVO.setMngrPageCrs(adminPageCrs);
                bbsRouterVO.setCompnCrs(compnCrs);
                bbsRouterVO.setExpsrYn(vo.getExpsrYn());

                // 게시판 라우터 등록
                result += saveBbsRouter(bbsRouterVO);
            }

            if(result != bbsTypeList.size()) {
                throw new Exception("게시판 라우터 등록에 실패했습니다.");
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("게시판 라우터 등록 중 오류가 발생했습니다.");
        }
    }
    /** ********************************* 게시판 라우터 (종료) ********************************* */
}