package com.takensoft.cms.cntnStats.service.Impl;

import com.takensoft.cms.bbs.dao.BbsMngDAO;
import com.takensoft.cms.cntnStats.dao.CntnStatsDAO;
import com.takensoft.cms.cntnStats.dto.CntnStatsDTO;
import com.takensoft.cms.cntnStats.service.CntnStatsService;
import com.takensoft.cms.cntnStats.vo.CntnStatsVO;
import com.takensoft.cms.menu.dao.MenuDAO;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.util.*;

/**
 * @author  : takensoft
 * @since   : 2024.05.23
 *
 * 접속통계 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * CntnStatsService : 접속 통계 관련 인터페이스 상속
 */
@Service("cntnStatsService")
@RequiredArgsConstructor
public class CntnStatsServiceImpl extends EgovAbstractServiceImpl implements CntnStatsService {

    private static final Logger LOGGER = LoggerFactory.getLogger(CntnStatsServiceImpl.class);
    private final CntnStatsDAO cntnStatsDAO;
    private final BbsMngDAO bbsMngDAO;
    private final MenuDAO menuDAO;

    /**
     * @author  : takensoft
     * @since   : 2024.05.23
     *
     * 접속 통계 등록
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int cntnStatsSave(CntnStatsDTO cntnStatsDTO) throws Exception {
        LocalDate date = LocalDate.now(); // 오늘 금일 날짜 출력
        CntnStatsVO vo = new CntnStatsVO(); // 접속 통계 객체 생성

        vo.setCntnYr(String.valueOf(date.getYear())); // 년도 추출
        vo.setCntnMm(String.format("%02d", date.getMonthValue())); // 월 추출
        vo.setCntnDay(String.format("%02d", date.getDayOfMonth())); // 일 추출

        int result = 0;

        // 사용자 접속 통계 생성
        if(cntnStatsDTO.getTypeId() == null || cntnStatsDTO.getTypeId().equals("")) {
            result += userCntnStatsSave(vo, cntnStatsDTO);
        // 메뉴 접속 통계 생성
        } else if(cntnStatsDTO.getTypeId().contains("MENU")) {
            result += menuCntnStatsSave(vo, cntnStatsDTO);
        // 게시판 접속 통계 생성
        } else if(cntnStatsDTO.getTypeId().contains("BBS")) {
            result += bbsCntnStatsSave(vo, cntnStatsDTO);
        // 콘텐츠 접속 통게 생성
        } else if(cntnStatsDTO.getTypeId().contains("CONTS")) {
            result += contsCntnStatsSave(vo, cntnStatsDTO);
        }

        return result;
    }

    /**
     * @author takensoft
     * @since 2024.05.23
     * 사용자 접속 통계 등록
     */
    private int userCntnStatsSave(CntnStatsVO cntnStatsVO, CntnStatsDTO cntnStatsDTO) throws Exception {
        int result = 0;
        for(String auth : cntnStatsDTO.getMbrAuthList()) {
            cntnStatsVO.setAuthrtCd(auth);
            boolean chck = cntnStatsDAO.userCntnStatsSaveChck(cntnStatsVO);
            // 카운트 증가
            if(chck) {
                result += cntnStatsDAO.userCntnStatsUpdate(cntnStatsVO);
            // 신규 등록
            } else {
                result += cntnStatsDAO.userCntnStatsSave(cntnStatsVO);
            }
        }
        return result;
    };

    /**
     * @author takensoft
     * @since 2024.05.23
     * 컨텐츠 접속 통계 등록
     */
    private int contsCntnStatsSave(CntnStatsVO cntnStatsVO, CntnStatsDTO cntnStatsDTO)throws Exception {
        int result = 0;
        for(String auth : cntnStatsDTO.getMbrAuthList()) {
            cntnStatsVO.setAuthrtCd(auth);
            cntnStatsVO.setTypeId(cntnStatsDTO.getTypeId());
            boolean chck = cntnStatsDAO.contsCntnStatsSaveChck(cntnStatsVO);
            // 카운트 증가
            if(chck) {
                result += cntnStatsDAO.contsCntnStatsUpdate(cntnStatsVO);
                // 신규 등록
            } else {
                result += cntnStatsDAO.contsCntnStatsSave(cntnStatsVO);
            }
        }
        return result;
    };

    /**
     * @author takensoft
     * @since 2024.05.23
     * 게시판 접속 통계 등록
     */
    private int bbsCntnStatsSave(CntnStatsVO cntnStatsVO, CntnStatsDTO cntnStatsDTO)throws Exception {
        int result = 0;
        for(String auth : cntnStatsDTO.getMbrAuthList()) {
            cntnStatsVO.setAuthrtCd(auth);
            cntnStatsVO.setTypeId(cntnStatsDTO.getTypeId());
            boolean chck = cntnStatsDAO.bbsCntnStatsSaveChck(cntnStatsVO);
            // 카운트 증가
            if(chck) {
                result += cntnStatsDAO.bbsCntnStatsUpdate(cntnStatsVO);
                // 신규 등록
            } else {
                result += cntnStatsDAO.bbsCntnStatsSave(cntnStatsVO);
            }
        }
        return result;
    };

    /**
     * @author takensoft
     * @since 2024.05.23
     * 메뉴 접속 통계 등록
     */
    private int menuCntnStatsSave(CntnStatsVO cntnStatsVO, CntnStatsDTO cntnStatsDTO)throws Exception {
        int result = 0;
        for(String auth : cntnStatsDTO.getMbrAuthList()) {
            cntnStatsVO.setAuthrtCd(auth);
            cntnStatsVO.setTypeId(cntnStatsDTO.getTypeId());
            boolean chck = cntnStatsDAO.menuCntnStatsSaveChck(cntnStatsVO);
            // 카운트 증가
            if(chck) {
                result += cntnStatsDAO.menuCntnStatsUpdate(cntnStatsVO);
                // 신규 등록
            } else {
                result += cntnStatsDAO.menuCntnStatsSave(cntnStatsVO);
            }
        }
        return result;
    };

    /**
     * @author 박정하
     * @since 2024.05.29
     *
     * 사용자 접속 통계
     */
    @Override
    public HashMap<String, Object> userCntnStatsSelect(HashMap<String, Object> params) throws Exception {
        HashMap<String, Object> result = new HashMap<>();
        result.put("list", cntnStatsDAO.selectUserCntnStats(params));
        result.put("totalCnt", cntnStatsDAO.selectUserCntnStatsCnt(params));
        return result;
    };

    /**
     * @author 박정하
     * @since 2024.05.30
     *
     * 메뉴별 접속 통계
     */
    @Override
    public HashMap<String, Object> menuCntnStatsSelect(HashMap<String, Object> params) throws Exception {
        HashMap<String, Object> result = new HashMap<>();
        int menuCnt = menuDAO.findAllCnt(params);
        result.put("menuCnt", menuCnt);
        if (menuCnt > 0) {
            result.put("list", cntnStatsDAO.selectMenuCntnStats(params));
            result.put("totalCnt", cntnStatsDAO.selectMenuCntnStatsCnt(params));
        }
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.05.30
     *
     * 게시판 접속 통계
     */
    @Override
    public HashMap<String, Object> bbsCntnStatsSelect(HashMap<String, Object> params) throws Exception {
        HashMap<String, Object> result = new HashMap<>();
        int bbsCnt = bbsMngDAO.findAllCntNotPagination();
        result.put("bbsCnt", bbsCnt);
        if (bbsCnt > 0) {
            result.put("list", cntnStatsDAO.selectBbsCntnStats(params));
            result.put("totalCnt", cntnStatsDAO.selectBbsCntnStatsCnt(params));
        }
        return result;
    }
}