package com.takensoft.cms.dept.service.Impl;

import com.takensoft.cms.dept.dao.DeptDAO;
import com.takensoft.cms.dept.service.DeptService;
import com.takensoft.cms.dept.vo.DeptAuthrtVO;
import com.takensoft.cms.dept.vo.DeptMbrVO;
import com.takensoft.cms.dept.vo.DeptVO;
import com.takensoft.common.HierachyVO;
import com.takensoft.common.idgen.service.IdgenService;
import com.takensoft.common.util.JWTUtil;
import com.takensoft.common.util.Secret;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author  : takensoft
 * @since   : 2024.04.24
 *
 * 부서 정보 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * DeptService : 부서 정보 인터페이스 상속
 */
@Service("deptService")
@RequiredArgsConstructor
public class DeptServiceImpl extends EgovAbstractServiceImpl implements DeptService {

    private static final Logger LOGGER = LoggerFactory.getLogger(DeptServiceImpl.class);
    private final JWTUtil jwtUtil;
    private final DeptDAO deptDAO;
    private final IdgenService deptIdgn;

    /**
     * @author takensoft
     * @since 2024.04.25
     * 부서 등록
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public HashMap<String, Object> deptSave(DeptVO deptVO) throws Exception {
        String deptId = deptIdgn.getNextStringId(); // 부서 아이디 생성
        deptVO.setDeptId(deptId);

        // 상위 부서가 존재하는 경우
        if(deptVO.getUpDeptId() != null && !deptVO.getUpDeptId().equals("")) {
            // 상위 부서보다 깊이(레벨) + 1
            deptVO.setDeptGrd(deptDAO.findByDeptGrd(deptVO.getUpDeptId()) + 1);
            // 동일한 깊이의 부서 순서보다 + 1
            deptVO.setDeptSn(deptDAO.findByDeptSn(deptVO.getUpDeptId()) + 1);
        // 상위 부서가 존재 하지 않는 경우
        } else {
            deptVO.setDeptGrd(0);
            deptVO.setDeptSn(deptDAO.findByDeptSn(null) + 1);
        }
        deptVO.setRgtr(jwtUtil.getWriter()); // 작성자 정보 등록
        int insertResult = deptDAO.save(deptVO);
        // 부서 권한 등록
        if(deptVO.getAuthrtList().size() > 0) {
            for(DeptAuthrtVO vo : deptVO.getAuthrtList()) {
                vo.setDeptId(deptId);
                vo.setRgtr(jwtUtil.getWriter());
                insertResult += deptDAO.authrtSave(vo);
            }
        }
        HashMap<String, Object> result = new HashMap<>();
        result.put("deptId", deptId);
        result.put("insertResult", insertResult); // 부서 등록
        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.29
     * 부서 사용자 등록
     */
    @Override
    public int deptMbrSave(DeptMbrVO deptMbrVO) throws Exception {
        deptMbrVO.setRgtr(jwtUtil.getWriter()); // 작성자 정보 등록
        return deptDAO.deptMbrSave(deptMbrVO);
    }

    /************************************** Hierachy 전용 **************************************/
    /**
     * @author takensoft
     * @since 2024.04.25
     * 최상위 부서 조회
     */
    @Override
    public List<HierachyVO> findByTopNode() throws Exception {
        List<HierachyVO> topDeptList = deptDAO.findByTopNode();
        for(HierachyVO vo : topDeptList) {
            List<HierachyVO> child = findChildNode(vo.getId());
            vo.setChildList(child);
        }
        return topDeptList;
    }

    /**
     * @author takensoft
     * @since 2024.04.25
     * 하위 부서 조회
     */
    @Override
    public List<HierachyVO> findChildNode(String id) throws Exception {
        List<HierachyVO> childList = deptDAO.findChildNode(id);
        for(HierachyVO vo : childList) {
            List<HierachyVO> child = findChildNode(vo.getId());
            vo.setChildList(child);
        }
        return childList;
    }
    /************************************** Hierachy 전용 **************************************/

    /**
     * @author takensoft
     * @since 2024.04.26
     * 부서 정보 조회
     */
    @Override
    public Map<String, Object> findByDept(HashMap<String, Object> params) throws Exception {
        String deptId = params.get("deptId").toString(); // 아이디 추출
        DeptVO dept = deptDAO.findByDept(deptId); // 부서 정보 조회
        List<DeptMbrVO> deptMbr = deptDAO.findByDeptMber(deptId); // 부서에 등록된 사용자 정보 조회
        // 사용자 정보 복호화
        if(deptMbr.size() > 0) {
            for(DeptMbrVO vo : deptMbr) {
                // 연락처 복호화
                vo.setMblTelno(Secret.decrypt(vo.getMblTelno().toString()));
            }
        }
        Map<String, Object> result = new HashMap<String, Object>(); // 반환 객체 생성
        result.put("dept", dept); // 부서 상세 정보
        result.put("deptMbr", deptMbr); // 부서 사용자 목록 정보
        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.26
     * 부서 정보 수정
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deptUpdate(DeptVO deptVO) throws Exception {
        deptVO.setMdfr(jwtUtil.getWriter()); // 토큰에서 사용자 정보 추출
        int result = deptDAO.update(deptVO); // 부서정보 수정
        result += deptDAO.deleteAuth(deptVO.getDeptId()); // 부서 권한 삭제
        // 부서 권한 등록
        if(deptVO.getAuthrtList().size() > 0) {
            for(DeptAuthrtVO vo : deptVO.getAuthrtList()) {
                vo.setDeptId(deptVO.getDeptId());
                vo.setRgtr(jwtUtil.getWriter());
                result += deptDAO.authrtSave(vo);
            }
        }
        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.26
     * 부서 정보 삭제
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deptDelete(DeptVO deptVO) throws Exception {
        int result = deptDAO.deleteDept(deptVO.getDeptId());
        result += deptDAO.deleteDeptInDeptMbr(deptVO.getDeptId());
        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.26
     * 부서 사용자 삭제
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deptMbrDelete(List<DeptMbrVO> deptMbrList) throws Exception {
        int result = 0;
        if(deptMbrList.size() == 0) return 0;
        for(DeptMbrVO vo : deptMbrList) {
            Map<String, Object> param = new HashMap<String, Object>();
            param.put("mbrId", vo.getMbrId());
            result += deptDAO.deleteDeptMbr(param);
        }
        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.29
     * 부서에 등록되지 않는 사용자 조회
     */
    @Override
    public Map<String, Object> findByMbr(Map<String, String> params) throws Exception {
        List<DeptMbrVO> mbrList = deptDAO.findByMber(params); // 부서에 등록되지 않는 사용자 조회
        // 사용자 정보 복호화
        if(mbrList.size() > 0) {
            for(DeptMbrVO vo : mbrList) {
                // 연락처 복호화
                vo.setMblTelno(Secret.decrypt(vo.getMblTelno().toString()));
            }
        }
        List<DeptMbrVO> deptMbr = deptDAO.findByDeptMber(params.get("deptId").toString()); // 부서에 등록된 사용자 정보 조회
        // 사용자 정보 복호화
        if(deptMbr.size() > 0) {
            for(DeptMbrVO vo : deptMbr) {
                // 연락처 복호화
                vo.setMblTelno(Secret.decrypt(vo.getMblTelno().toString()));
            }
        }

        Map<String, Object> result = new HashMap<String, Object>(); // 반환 객체 생성
        result.put("mbrList", mbrList); // 부서에 등록되지 않는 사용자 정보
        result.put("deptMbr", deptMbr); // 부서 사용자 목록 정보
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.05.10
     * 부서 정보 수정 (hierachyVO 사용)
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateList(List<HierachyVO> deptList) throws Exception {
        int result = 0;
        String writer = jwtUtil.getWriter();

        if(deptList != null && deptList.size() > 0) {
            int sn = 1;
            for(HierachyVO hierachyVO : deptList) {
                // 최상위 노드 수정
                DeptVO vo = new DeptVO();
                vo.setDeptId(hierachyVO.getId());
                vo.setUpDeptId(null);
                vo.setDeptGrd(0);
                vo.setDeptSn(sn);
                vo.setMdfr(writer);
                result += deptDAO.deptUpdateByHierachy(vo);

                // 하위 노드 수정
                if (hierachyVO.getChildList() != null && hierachyVO.getChildList().size() > 0) {
                    result += updateChildNode(hierachyVO.getChildList(), vo);
                }
                sn++;
            }
        }
        return result;
    }
    /**
     * @author 박정하
     * @since 2024.05.10
     * 하위 부서 수정
     */
    @Override
    public int updateChildNode(List<HierachyVO> childList, DeptVO upDeptVO) throws Exception {
        int result = 0;
        String upDeptId = upDeptVO.getDeptId();
        int grd = upDeptVO.getDeptGrd() + 1;
        int sn = 1;
        String mdfr = upDeptVO.getMdfr();
        for(HierachyVO hierachyVO : childList) {
            // 현재 노드 수정
            DeptVO vo = new DeptVO();
            vo.setDeptId(hierachyVO.getId());
            vo.setUpDeptId(upDeptId);
            vo.setDeptGrd(grd);
            vo.setDeptSn(sn);
            vo.setMdfr(mdfr);
            result += deptDAO.deptUpdateByHierachy(vo);

            // 하위 노드 수정
            if (hierachyVO.getChildList() != null && hierachyVO.getChildList().size() > 0) {
                result += updateChildNode(hierachyVO.getChildList(), vo);
            }
            sn++;
        }
        return result;
    }
}