package com.takensoft.common.filter;

import com.takensoft.cms.accesCtrl.service.AccesCtrlService;
import com.takensoft.cms.accesCtrl.service.SystemAccesCtrlService;
import com.takensoft.cms.accesCtrl.vo.AccesCtrlVO;
import com.takensoft.common.config.CommonConfig;
import com.takensoft.common.util.CommonUtils;
import com.takensoft.common.util.ErrorResponse;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @author takensoft
 * @since 2024.04.15
 *
 * Access(아이피) 검증 필터
 */
public class AccesFilter extends OncePerRequestFilter {

    private final AccesCtrlService accesCtrlService;
    private final CommonUtils commonUtils;
    private final CommonConfig commonConfig;

    public AccesFilter(AccesCtrlService accesCtrlService, CommonUtils commonUtils, CommonConfig commonConfig) {
        this.accesCtrlService = accesCtrlService;
        this.commonUtils = commonUtils;
        this.commonConfig = commonConfig;
    }

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        try {
            // 아이피 정보
            String ipAdrs = commonUtils.getIp(request);
            // 접근 제어 정보 조회
            List<AccesCtrlVO> accesCtrlList = accesCtrlService.findAllAccesCtrlSecurity(ipAdrs);

            // 접근 제어 정보가 없거나 접근 제어가 허용된 경우 다음 필터로 이동
            if(accesCtrlList.isEmpty() || isAccesCntrl(accesCtrlList, request)) {
                filterChain.doFilter(request, response);
                return;
            }
            // 접근 제어가 제한된 경우 에러 발생
            ErrorResponse errorResponse = new ErrorResponse();
            errorResponse.setMessage("접근 권한이 없습니다.");  // 에러 메시지 설정
            errorResponse.setPath(request.getRequestURI()); // 에러 경로 설정
            errorResponse.setError(HttpStatus.FORBIDDEN.getReasonPhrase()); // 에러 원인 설정
            errorResponse.setStatus(HttpStatus.FORBIDDEN.value()); // 에러 상태코드 설정
            errorResponse.setTimestamp(LocalDateTime.now()); // 에러 응답 시간 설정

            // 응답 헤더 설정 및 json 응답 전송
            response.setContentType(MediaType.APPLICATION_JSON_VALUE);
            response.setStatus(HttpStatus.FORBIDDEN.value());
            response.getOutputStream().write(commonConfig.getObjectMapper().writeValueAsBytes(errorResponse));
        } catch (Exception e) {
            // 시스템 에러 발생
            e.printStackTrace();
            ErrorResponse errorResponse = new ErrorResponse();
            errorResponse.setMessage("Server Internal Error"); // 에러 메시지 설정
            errorResponse.setPath(request.getRequestURI()); // 에러 경로 설정
            errorResponse.setError(HttpStatus.INTERNAL_SERVER_ERROR.getReasonPhrase());  // 에러 원인 설정
            errorResponse.setStatus(HttpStatus.INTERNAL_SERVER_ERROR.value()); // 에러 상태코드 설정
            errorResponse.setTimestamp(LocalDateTime.now()); // 에러 응답 시간 설정

            // 응답 헤더 설정 및 json 응답 전송
            response.setContentType(MediaType.APPLICATION_JSON_VALUE);
            response.setStatus(HttpStatus.INTERNAL_SERVER_ERROR.value());
            response.getOutputStream().write(commonConfig.getObjectMapper().writeValueAsBytes(errorResponse));
        }
    }

    // 요청 URI에 따른 접근 제어 확인
    private boolean isAccesCntrl(List<AccesCtrlVO> accesCtrlList, HttpServletRequest req) {
        for(AccesCtrlVO vo : accesCtrlList) {
            AntPathRequestMatcher matcher = new AntPathRequestMatcher(vo.getCntrlCrs());
            // 접근 제어 경로에 해당 하는 경우
            if(matcher.matches(req)) {
                return "permit".equals(vo.getCntrlType()); // 허용일 경우 true 반환, 제한일 경우 false 반환
            }
        }
        return true;  // 접근 제어 목록에 요청 경로가 없는 경우 허용으로 처리
    }
}
