package com.takensoft.common.filter;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.takensoft.cms.mber.vo.MberAuthorVO;
import com.takensoft.cms.mber.vo.MberVO;
import com.takensoft.common.config.CommonConfig;
import com.takensoft.common.util.ErrorResponse;
import com.takensoft.common.util.JWTUtil;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.JwtException;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.authentication.InsufficientAuthenticationException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @author takensoft
 * @since 2024.04.01
 *
 * JWT - Access 토큰 검증 필터
 */
public class JWTFilter extends OncePerRequestFilter {

    private final JWTUtil jwtUtil;
    private final CommonConfig commonConfig;
    public JWTFilter(JWTUtil jwtUtil, CommonConfig commonConfig) {
        this.jwtUtil = jwtUtil;
        this.commonConfig = commonConfig;
    }

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        // 헤더에서 access에 대한 토큰을 꺼냄
        String accessToken = request.getHeader("Authorization");
        // 토큰이 없다면 다음 필터로 넘김
        if(accessToken == null) {
            filterChain.doFilter(request, response);
            return;
        }
        try {
            // 토큰 만료 여부 검증
            if(jwtUtil.isExpired(accessToken)) {
                throw new JwtException("Token expired");
            }
            // 토큰에서 페이로드 확인[ 발급시 명시 ]
            String category = jwtUtil.getCategory(accessToken);
            if(!category.equals("Authorization")) {
                throw new JwtException("Wrong Token");
            }
            // 토큰에서 사용자 정보 추출
            MberVO mber = new MberVO();
            List<MberAuthorVO> roles = jwtUtil.getRoles(accessToken);
            mber.setLgnId(jwtUtil.getLgnId(accessToken));
            mber.setMbrId(jwtUtil.getMbrId(accessToken));
            mber.setAuthorList(roles);

            // 시큐리티 컨텍스트에 사용자 정보 설정
            Authentication authToken = new UsernamePasswordAuthenticationToken(mber, null, mber.getAuthorities());
            SecurityContextHolder.getContext().setAuthentication(authToken);
            // 다음 필터로 이동
            filterChain.doFilter(request, response);
        } catch(JwtException | InsufficientAuthenticationException e) {
            // 토큰 검증 실패 시 클라이언트에게 에러 응답을 위한 객체 생성
            ErrorResponse errorResponse = new ErrorResponse();
            // 에러 응답 메시지 설정
            if(e instanceof ExpiredJwtException) {
                errorResponse.setMessage("Token expired");
            } else {
                errorResponse.setMessage(e.getMessage());
            }
            errorResponse.setPath(request.getRequestURI()); // 오류 경로 설정
            errorResponse.setError(HttpStatus.UNAUTHORIZED.getReasonPhrase()); // 오류 원인 설정
            errorResponse.setStatus(HttpStatus.UNAUTHORIZED.value()); // 상태코드 설정
            errorResponse.setTimestamp(LocalDateTime.now()); // 응답 시간 설정

            // 응답 헤더 설정 및 json 응답 전송
            response.setContentType(MediaType.APPLICATION_JSON_VALUE);
            response.setStatus(HttpStatus.UNAUTHORIZED.value());
            response.getOutputStream().write(commonConfig.getObjectMapper().writeValueAsBytes(errorResponse));
        }
    }
}