package com.takensoft.portal.entDscsnAply.service.Impl;

import com.takensoft.cms.mber.vo.MberAuthorVO;
import com.takensoft.common.Pagination;
import com.takensoft.common.file.service.FileMngService;
import com.takensoft.common.file.service.FileService;
import com.takensoft.common.file.vo.FileMngVO;
import com.takensoft.common.idgen.service.IdgenService;
import com.takensoft.common.util.CommonUtils;
import com.takensoft.common.util.JWTUtil;
import com.takensoft.portal.entDscsnAply.dao.EntDscsnAplyDAO;
import com.takensoft.portal.entDscsnAply.service.EntDscsnAplyService;
import com.takensoft.portal.entDscsnAply.vo.EntDscsnAplyVO;
import com.takensoft.ums.service.UmsService;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.04.02
 *
 * 기업상담신청 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * EntDscsnAplyService : 기업상담신청 인터페이스 상속
 */
@Service("entDscsnAplyService")
@RequiredArgsConstructor
public class EntDscsnAplyServiceImpl extends EgovAbstractServiceImpl implements EntDscsnAplyService {
    private final JWTUtil jwtUtil;
    private final EntDscsnAplyDAO entDscsnAplyDAO;
    private final FileMngService fileMngService;
    private final IdgenService entDscsnAplyIdgn;
    private final FileService fileService;
    private final CommonUtils commonUtils;
    private final UmsService umsService;

    /**
     * @author 박정하
     * @since 2024.04.02
     *
     * 기업상담신청 등록
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public HashMap<String, Object> entDscsnAplyInsert(HttpServletRequest req, EntDscsnAplyVO entDscsnAplyVO) throws Exception {
        // 아이피 생성
        String ipAddress = commonUtils.getIp(req);
        entDscsnAplyVO.setAplcntIp(ipAddress);

        // 아이피 검사
        HashMap<String, Object> result = new HashMap<>();
        result.put("insertResult", 0);
        HashMap<String, Object> entDscsnAplyCount = entDscsnAplyDAO.entDscsnAplyCountByOneHourWriteSameIp(ipAddress);
        if (Integer.parseInt((String.valueOf(entDscsnAplyCount.get("writecount")))) > 2) {
            result.put("writeCount", entDscsnAplyCount.get("writecount"));
            result.put("lastDatetime", entDscsnAplyCount.get("lastdatetime"));
            return result;
        }

        // 기업상담신청 아이디 생성
        String entDscsnAplyId = entDscsnAplyIdgn.getNextStringId();
        entDscsnAplyVO.setEntDscsnAplyId(entDscsnAplyId);

        // 첨부파일
        List<MultipartFile> insertFileList = entDscsnAplyVO.getInsertFileList();
        if (insertFileList.size() > 0) {
            FileMngVO fileMngVO = new FileMngVO();

            // 파일매니저 등록
            HashMap<String, Object> fileMng = fileMngService.fileMngInsert(fileMngVO, insertFileList);

            // 파일매니저 아이디 삽입
            entDscsnAplyVO.setFileMngId(fileMng.get("fileMngId").toString());
        }

        // 기업상담신청 등록
        int insertResult = entDscsnAplyDAO.entDscsnAplyInsert(entDscsnAplyVO);

        // 기업 상담 등록 시 문자 메시지 발송
        Map<String, Object> umsResult = umsService.save();

        // 결과 반환
        result.put("insertResult", insertResult);
        result.put("umsResult", umsResult);
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.04.02
     *
     * 기업상담신청 목록 조회
     */
    @Override
    public Map<String, Object> entDscsnAplyList(HttpServletRequest request, Map<String, String> params) throws Exception {
        String accessToken = request.getHeader("Authorization");
        List<MberAuthorVO> roles = jwtUtil.getRoles(accessToken);
        if (roles.size() > 0) {
            for (MberAuthorVO role : roles) {
                if (role.getAuthrtCd().equals("ROLE_ADMIN")){
                    params.put("isAdmin", "Y");
                }
            }
        }
        params.put("mbrId", jwtUtil.getWriter());

        Pagination search = new Pagination(0, params);
        int cnt = entDscsnAplyDAO.entDscsnAplyListCount(search);
        Pagination pagination = new Pagination(cnt, params);
        List<EntDscsnAplyVO> list = entDscsnAplyDAO.entDscsnAplyList(pagination);

        Map<String, Object> result = new HashMap<>();
        result.put("list", list);
        result.put("pagination", pagination);
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.04.02
     *
     * 기업상담신청 상세 조회
     */
    @Override
    public EntDscsnAplyVO entDscsnAplyDetail(EntDscsnAplyVO entDscsnAplyVO) throws Exception {
        EntDscsnAplyVO newEntDscsnAplyVO = new EntDscsnAplyVO();
        
        // entDscsnAplyId가 있는 경우
        if (entDscsnAplyVO.getEntDscsnAplyId() != null) {
            newEntDscsnAplyVO = entDscsnAplyDAO.entDscsnAplyDetail(entDscsnAplyVO.getEntDscsnAplyId());

            // FileMngId가 있는 경우
            if (newEntDscsnAplyVO.getFileMngId() != null) {
                List<HashMap<String, Object>> fileList = fileService.fileSelectList(newEntDscsnAplyVO.getFileMngId());
                newEntDscsnAplyVO.setFileList(fileList);
            }            
        }
        // entDscsnAplyId가 없는 경우
        else {
            newEntDscsnAplyVO.setUseYn("Y"); // 사용유무
        }

        return newEntDscsnAplyVO;
    }

    /**
     * @author 박정하
     * @since 2024.04.02
     *
     * 기업상담신청 수정
     */
    @Override
    public int entDscsnAplyUpdate(EntDscsnAplyVO entDscsnAplyVO) throws Exception {
        return entDscsnAplyDAO.entDscsnAplyUpdate(entDscsnAplyVO);
    }
}