package com.takensoft.portal.entInfo.service.Impl;

import com.takensoft.cms.codeManage.service.CodeManageService;
import com.takensoft.cms.codeManage.vo.CodeManageVO;
import com.takensoft.common.Pagination;
import com.takensoft.common.file.service.FileMngService;
import com.takensoft.common.file.service.FileService;
import com.takensoft.common.file.vo.FileMngVO;
import com.takensoft.common.idgen.service.IdgenService;
import com.takensoft.common.util.JWTUtil;
import com.takensoft.portal.entInfo.dao.EntInfoDAO;
import com.takensoft.portal.entInfo.dto.EntInfoExcelDTO;
import com.takensoft.portal.entInfo.service.EntInfoService;
import com.takensoft.portal.entInfo.vo.EntInfoVO;
import com.takensoft.portal.entInfo.vo.BplcVO;
import com.takensoft.portal.ivstDscsn.service.IvstDscsnService;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.03.20
 *
 * 기업정보 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * EntInfoService : 기업정보 인터페이스 상속
 */
@Service("entInfoService")
@RequiredArgsConstructor
public class EntInfoServiceImpl extends EgovAbstractServiceImpl implements EntInfoService {
    private final EntInfoDAO entInfoDAO;
    private final IdgenService entInfoIdgn;
    private final IvstDscsnService ivstDscsnService;
    private final JWTUtil jwtUtil;
    private final BplcServiceImpl entMainBplcService;
    private final CodeManageService codeManageService;
    private final FileMngService fileMngService;
    private final FileService fileService;

    /**
     * @author 박정하
     * @since 2024.03.20
     *
     * 기업정보 등록
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public HashMap<String, Object> entInfoInsert(EntInfoVO entInfoVO, List<MultipartFile> insertFileList) throws Exception {
        // 기업 아이디 생성
        String entId = entInfoIdgn.getNextStringId();
        entInfoVO.setEntId(entId);

        // 작성자 생성
        String writer = jwtUtil.getWriter();
        entInfoVO.setRgtr(writer);

        // 첨부파일 등록
        if (insertFileList != null && !insertFileList.isEmpty() && insertFileList.size() > 0) {
            FileMngVO fileMngVO = new FileMngVO();

            // 파일매니저 등록
            HashMap<String, Object> fileMng = fileMngService.fileMngInsert(fileMngVO, insertFileList);

            // 파일매니저 아이디 삽입
            entInfoVO.setFileMngId(fileMng.get("fileMngId").toString());
        }

        // 기업정보 등록
        int insertResult = entInfoDAO.entInfoInsert(entInfoVO);

        // 주요사업장 등록
        if (entInfoVO.getBplcList().size() > 0) {
            List<BplcVO> newBplcList = new ArrayList<>();
            for (BplcVO bplcVO : entInfoVO.getBplcList()) {
                bplcVO.setEntId(entId);
                bplcVO.setRgtr(writer);
                newBplcList.add(bplcVO);
            }
            insertResult += entMainBplcService.bplcListInsert(newBplcList);
        }

        // 기업정보 등록
        HashMap<String, Object> result = new HashMap<>();
        result.put("entId", entId);
        result.put("result", insertResult);
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.03.21
     *
     * 사업자등록번호 중복 검사
     */
    @Override
    public int selectCntByCheckBrno(String brno) throws Exception {
        return entInfoDAO.selectCntByCheckBrno(brno);
    }

    /**
     * @author 박정하
     * @since 2024.03.21
     *
     * 기업정보 목록 조회
     */
    @Override
    public Map<String, Object> entInfoSelectList(Map<String, String> params) throws Exception {
        Pagination search = new Pagination(0, params);
        int cnt = entInfoDAO.entInfoSelectListCount(search);
        Pagination pagination = new Pagination(cnt, params);
        List<EntInfoVO> list = entInfoDAO.entInfoSelectList(pagination);

        // 상위업종코드 목록 조회
        List<String> ksicUpCdList = entInfoDAO.ksicUpCdList();

        // 결과 반환
        Map<String, Object> result = new HashMap<>();
        result.put("list", list);
        result.put("pagination", pagination);
        result.put("ksicUpCdList", ksicUpCdList);
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.03.21
     *
     * 기업정보 상세 조회
     */
    @Override
    public HashMap<String, Object> entInfoDetail(EntInfoVO entInfoVO) throws Exception {
        EntInfoVO newEntInfoVO = new EntInfoVO();

        // entInfoId가 있는 경우
        if (entInfoVO.getEntId() != null) {
            newEntInfoVO = entInfoDAO.selectEntInfoDetail(entInfoVO.getEntId());
            // FileMngId가 있는 경우
            if (newEntInfoVO.getFileMngId() != null) {
                List<HashMap<String, Object>> fileList = fileService.fileSelectList(newEntInfoVO.getFileMngId());
                newEntInfoVO.setFileList(fileList);
            }
        }
        // entInfoId가 없는 경우
        else {
            newEntInfoVO.setUseYn("Y"); // 사용유무
            newEntInfoVO.setIsBrnoChange(false); // 사업자번호 변경 여부
        }

        // 주요사업장
        if (newEntInfoVO.getBplcList().size() == 0) {
            BplcVO bplcVO = new BplcVO();
            bplcVO.setEntId(newEntInfoVO.getEntId()); // 기업정보 Id

            List<BplcVO> bplcList = new ArrayList<>();
            bplcList.add(bplcVO);

            newEntInfoVO.setBplcList(bplcList);
        }

        // 면적 단위
        List<CodeManageVO> bplcCodeList = codeManageService.findByChildCdCache("areaUnit");
        List<CodeManageVO> lctnCodeList = codeManageService.findByChildCdCache("admdsp");

        HashMap<String, Object> result = new HashMap<>();
        result.put("entInfoVO", newEntInfoVO);
        result.put("bplcCodeList", bplcCodeList);
        result.put("lctnCodeList", lctnCodeList);
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.03.22
     *
     * 기업정보 수정
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int entInfoUpdate(EntInfoVO entInfoVO, List<MultipartFile> insertFileList) throws Exception {
        // 수정자 생성
        String writer = jwtUtil.getWriter();
        entInfoVO.setMdfr(writer);

        int result = 0;

        // 첨부파일 정보
        String fileMngId = entInfoVO.getFileMngId();
        FileMngVO fileMngVO = new FileMngVO();
        List<HashMap<String, Object>> deleteFileList = entInfoVO.getDeleteFileList();

        // FileMngId가 있는 경우
        if (fileMngId != null) {
            fileMngVO.setFileMngId(entInfoVO.getFileMngId());
            for (HashMap<String, Object> file : deleteFileList) {
                file.put("fileId", Integer.parseInt(file.get("fileId").toString()));
            }
            result += fileMngService.fileMngUpadate(fileMngVO, deleteFileList, insertFileList);
        }
        // FileMngId가 없는 경우
        else {
            if (insertFileList != null && insertFileList.size() > 0) {
                HashMap<String, Object> fileMng = fileMngService.fileMngInsert(fileMngVO, insertFileList);
                result += (int)fileMng.get("fileMngResult");
                entInfoVO.setFileMngId(fileMng.get("fileMngId").toString());
            }
        }

        // 기업정보 수정
        result += entInfoDAO.entInfoUpdate(entInfoVO);

        // 주요사업장 수정
        if (entInfoVO.getBplcList().size() > 0) {
            for (BplcVO bplcVO : entInfoVO.getBplcList()) {
                bplcVO.setRgtr(writer);
            }
            result += entMainBplcService.bplcListUpdate(entInfoVO.getBplcList());
        }

        return result;
    }

    /**
     * @author 박정하
     * @since 2024.03.22
     *
     * 기업정보 삭제
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int entInfoDelete(EntInfoVO entInfoVO) throws Exception {
        // 수정자 생성
        String writer = jwtUtil.getWriter();
        entInfoVO.setMdfr(writer);

        // 기업정보 삭제
        int result = 0;
        entInfoVO.setUseYn("N");
        result += entInfoDAO.entInfoDelete(entInfoVO);

        // 주요사업장 삭제
        result += entMainBplcService.bplcDeleteByEntInfoId(entInfoVO.getEntId());

        // 투자상담 삭제
        result += ivstDscsnService.ivstDscsnDeleteByEntId(entInfoVO.getEntId());

        return result;
    }

    /**
     * @author 박정하
     * @since 2024.06.26
     *
     * 기업정보 목록 조회 (엑셀 다운로드 용)
     */
    public List<EntInfoExcelDTO> excelDownloadAll(HashMap<String, String> params) throws Exception {
        List<HashMap<String, String>> entInfoList = entInfoDAO.entInfoSelectListAll(params);

        List<EntInfoExcelDTO> entInfoExcelList = new ArrayList<>();
        for (HashMap<String, String> entInfo : entInfoList) {
            EntInfoExcelDTO entInfoExcelDTO = new EntInfoExcelDTO();
            entInfoExcelDTO.setEntNm(entInfo.get("ent_nm"));
            entInfoExcelDTO.setBrno(entInfo.get("brno"));
            entInfoExcelDTO.setRprsvNm(entInfo.get("rprsv_nm"));
            entInfoExcelDTO.setLctn(entInfo.get("lctn"));
            entInfoExcelDTO.setBplcSiar(entInfo.get("bplc_siar"));
            entInfoExcelDTO.setFctryArea(entInfo.get("fctry_area"));
            entInfoExcelDTO.setKsic(entInfo.get("ksic"));
            entInfoExcelDTO.setMainPrdct(entInfo.get("main_prdct"));
            entInfoExcelDTO.setEntCrdt(entInfo.get("ent_crdt"));
            entInfoExcelDTO.setFndnDt(entInfo.get("fndn_dt"));
            entInfoExcelDTO.setWrkrCnt(entInfo.get("wrkr_cnt"));
            entInfoExcelDTO.setIsIvstDscsn(entInfo.get("mvn_inten") != null ? "Y" : "N");
            entInfoExcelDTO.setMvnInten(convertInten(entInfo.get("mvn_inten")));
            entInfoExcelDTO.setMouInten(convertInten(entInfo.get("mou_inten")));
            entInfoExcelDTO.setRmrk(entInfo.get("rmrk"));

            entInfoExcelList.add(entInfoExcelDTO);
        }
        return entInfoExcelList;
    }

    /**
     * @author 박정하
     * @since 2024.06.26
     *
     * 의향 코드 > 한글
     */
    private String convertInten(String inten) {
        return inten == null ? null : inten.equals("0") ? "없음" : inten.equals("1") ? "있음" : inten.equals("2") ? "검토중" : null;
    }
}